==> nina-0.1.2-src.tar.gz: nina/java/translator/net/morilib/nina/translate/nina_template.dfa.java.sh <==
echo '// Translated by Nina'
echo '// This source code is under public domain'
[ -z "$PACKAGE" ] || echo "package $PACKAGE;"
print_imports

cat << EOF

public class $CLASSNAME {

	private int state;
	private int unread = -1;
	private String matched;

EOF
cat fragment
cat << EOF

	private int _read(java.io.Reader rd) throws java.io.IOException {
		int c;

		if(unread < 0) {
			return rd.read();
		} else {
			c = unread;
			unread = -1;
			return c;
		}
	}

	private boolean _step(int c) {
		switch(state) {
EOF

print_states
cat << EOF
		}
		return false;
	}

	private boolean _accepted() {
EOF

print_accepts
echo '	}'
echo
echo '	private void _execaction(int $c) {'
echo '		switch(state) {'

print_actions
cat << EOF
		}
	}

	public boolean matches(String s) {
		int n;

		state = 0;  matched = null;
		for(n = 0; n < s.length(); n++) {
			if(!_step(s.charAt(n))) {
				return false;
			}
		}

		if(_accepted()) {
			matched = s;
			return true;
		} else {
			return false;
		}
	}

	public boolean lookingAt(String s) {
		StringBuffer b = new StringBuffer();
		int n;

		state = 0;  matched = null;
		for(n = 0; n < s.length(); n++) {
			if(_accepted()) {
				matched = b.toString();
			}

			if(_step(s.charAt(n))) {
				// do nothing
			} else {
				return matched != null;
			}
			b.append(s.charAt(n));
		}

		if(_accepted()) {
			matched = b.toString();
			return true;
		} else {
			return matched != null;
		}
	}

	public boolean find(String s) {
		StringBuffer b;
		int n, k;

		state = 0;  matched = null;
		for(k = 0; k < s.length(); k++) {
			b = new StringBuffer();
			for(n = k; n < s.length(); n++) {
				if(_accepted()) {
					matched = b.toString();
				}

				if(_step(s.charAt(n))) {
					// do noting
				} else if(matched == null) {
					break;
				} else {
					return true;
				}
				b.append(s.charAt(n));
			}

			if(_accepted()) {
				matched = b.toString();
				return true;
			} else if(matched != null) {
				return true;
			} else {
				state = 0;
			}
		}
		return false;
	}

	public String group() {
		return matched;
	}

	public boolean parse(
			java.io.Reader rd) throws java.io.IOException {
		int p, c;

		for(p = -2; (c = _read(rd)) >= 0; p = c) {
			_execaction(p);
			if(!_step(c)) {
				throw new RuntimeException();
			}
		}
		_execaction(p);
		return _accepted();
	}

}
EOF

==> nina-0.1.3-src.tar.gz: nina/java/translator/net/morilib/nina/translate/nina_template.dfa.java.sh <==
echo '// Translated by Nina'
echo '// This source code is under public domain'
[ -z "$PACKAGE" ] || echo "package $PACKAGE;"
[ -z "$EXTENDS" ] || extends="extends $EXTENDS"
[ -z "$IMPLEMENTS" ] || implements="implements $IMPLEMENTS"
print_imports

cat << EOF

public $ABSTRACT class ${CLASSNAME}${TEMPLATE} $extends $implements {

	private int state;
EOF
if [ -z "$TYPE" ]
then
  echo "	private $CTYPE unread = -1;"
  echo '	private String matched;'
else
  echo "	private $CTYPE unread = null;"
fi

echo
cat fragment
echo

if [ -z "$TYPE" ]
then
  cat << EOF
	private int _read(java.io.Reader rd) throws java.io.IOException {
		int c;

		if(unread < 0) {
			return rd.read();
		} else {
			c = unread;
			unread = -1;
			return c;
		}
	}
EOF
else
  cat << EOF
	private $CTYPE _read(java.util.Iterator<$CTYPE> itr) {
		$CTYPE c;

		if(unread != null) {
			c = unread;
			unread = null;
			return c;
		} else if(itr.hasNext()) {
			return itr.next();
		} else {
			return null;
		}
	}
EOF
fi

echo
echo "	private boolean _step($CTYPE c) {"
echo '		switch(state) {'
print_states
echo '		}'
echo '		return false;'
echo '	}'
echo

echo '	private boolean _accepted() {'
print_accepts
echo '	}'
echo

echo "	private void _execaction($CTYPE" ' $c) {'
echo '		switch(state) {'
print_actions
echo '		}'
echo '	}'
echo

if [ -z "$TYPE" ]
then
  cat << EOF
	public boolean matches(String s) {
		int n;

		state = 0;  matched = null;
		for(n = 0; n < s.length(); n++) {
			if(!_step(s.charAt(n))) {
				return false;
			}
		}

		if(_accepted()) {
			matched = s;
			return true;
		} else {
			return false;
		}
	}

	public boolean lookingAt(String s) {
		StringBuffer b = new StringBuffer();
		int n;

		state = 0;  matched = null;
		for(n = 0; n < s.length(); n++) {
			if(_accepted()) {
				matched = b.toString();
			}

			if(_step(s.charAt(n))) {
				// do nothing
			} else {
				return matched != null;
			}
			b.append(s.charAt(n));
		}

		if(_accepted()) {
			matched = b.toString();
			return true;
		} else {
			return matched != null;
		}
	}

	public boolean find(String s) {
		StringBuffer b;
		int n, k;

		state = 0;  matched = null;
		for(k = 0; k < s.length(); k++) {
			b = new StringBuffer();
			for(n = k; n < s.length(); n++) {
				if(_accepted()) {
					matched = b.toString();
				}

				if(_step(s.charAt(n))) {
					// do noting
				} else if(matched == null) {
					break;
				} else {
					return true;
				}
				b.append(s.charAt(n));
			}

			if(_accepted()) {
				matched = b.toString();
				return true;
			} else if(matched != null) {
				return true;
			} else {
				state = 0;
			}
		}
		return false;
	}

	public String group() {
		return matched;
	}

	public boolean parse(
			java.io.Reader rd) throws java.io.IOException {
		int p, c;

		for(p = -2; (c = _read(rd)) >= 0; p = c) {
			_execaction(p);
			if(!_step(c)) {
				throw new RuntimeException();
			}
		}
		_execaction(p);
		return _accepted();
	}

}
EOF
else
  cat << EOF
	public boolean parse(Iterable<$CTYPE> x) {
		java.util.Iterator<$CTYPE> itr = x.iterator();
		$CTYPE p, c;

		for(p = null; (c = _read(itr)) != null; p = c) {
			_execaction(p);
			if(!_step(c)) {
				throw new RuntimeException();
			}
		}
		_execaction(p);
		return _accepted();
	}

}
EOF
fi
